#include <unistd.h>
#include <stdio.h>
#include <stdlib.h>
#include <pthread.h>
#include <signal.h>
#include <string.h>

#include "can_apply.h"


#define MASTR_CAN1_ID      0x100
#define MASTR_CAN2_ID      0x101

#define BACKUP_CAN1_ID     0x110
#define BACKUP_CAN2_ID     0x111

#define RX_MSG_NUM           16
#define MAX_BUF_LEN          64

typedef enum
{
	CAN_1,
	CAN_2,
	CAN_RES
}can_type;

typedef struct{
	int can_id;
	unsigned char can_dlc;
	unsigned char data[MAX_BUF_LEN];
}can_msg_t;


static int bexit = 0;
static int can0_sock,can1_sock;
can_msg_t can_rx_msg[RX_MSG_NUM];
unsigned int can1_id = 0;
unsigned int can2_id = 0;


void signal_handler(int signo)
{

    printf("signal %d(%s) received\n", signo, strsignal(signo));


	bexit = 1;

}

int can_send_data(can_type type, unsigned char *buf, int len)
{
    int i, j;
    int cnt;
    int sock;
    int ret = 0;
    struct can_frame frame;

    if(type == CAN_1)
    {
        sock = can0_sock;
        frame.can_id = can1_id;
    }
    else {
        sock = can1_sock;
        frame.can_id = can2_id;
    }

    for(i = 0; i < len; i += 8)
    {
        cnt = len - i;

        if(cnt > 8)
            frame.can_dlc =  8;
        else
            frame.can_dlc = cnt;

        for(j=0; j<frame.can_dlc; j++)
        {
            frame.data[j] = buf[i+j];
        }

        if(can_send(sock, frame) < 0)
        {
            ret = -1;
            break;
        }
    }

    return ret;
}

void can_recv_data(int sock)
{
    static unsigned char tail = 0;
    int i;
    int can_id;
    unsigned char can_dlc;
    unsigned char *data;
    struct can_frame frame;
    int index = can0_sock == sock ? 0 : 1;

    if(can_recv(sock, &frame) < 0)
        return ;


    can_id = frame.can_id;
    can_dlc = frame.can_dlc;
    data = frame.data;

    printf("CAN%d dlc = %d, can_id = %x\ndata:", index,frame.can_dlc, frame.can_id);
    for(i=0; i<frame.can_dlc; i++)
        printf("0x%02x ", frame.data[i]);
    printf("\n");


}

void *can_proc_start(void *arg)
{
    int nready;
    int maxfd;
    fd_set readfds;
    int sock = *(int *)arg;

    FD_ZERO(&readfds);
    FD_SET(sock, &readfds);
    maxfd = sock;
    while(!bexit)
    {
        nready = select(maxfd+1, &readfds, NULL, NULL, NULL);
        if(nready < 0)
        {
            perror("can select");
            break;
        }
        else if(nready == 0)
        {
            continue;
        }

        /* data is ready */
        if(FD_ISSET(sock, &readfds))
        {
            can_recv_data(sock);
        }
        else { ; }
    }

    close(sock);
    pthread_detach(pthread_self());
    pthread_exit(0);
}


static void can_sock_init()
{
    pthread_t tid;


    can0_sock = can_open("can0");
    if(can0_sock < 0)
        return ;

    can1_sock = can_open("can1");
    if(can1_sock < 0)
        return ;

    if (0 != pthread_create(&tid, NULL, can_proc_start, (void *)&can0_sock))
    {
        return ;
    }

    if (0 != pthread_create(&tid, NULL, can_proc_start, (void *)&can1_sock))
    {
        return ;
    }


    can1_id = MASTR_CAN1_ID;
    can2_id = MASTR_CAN2_ID;


    printf("can open sucess can0_sock:%d, can1_sock:%d\n", can0_sock, can1_sock);
}


int main(int argc, char *argv[])
{

    char *send_data = "hello,can!";
    signal(SIGINT, signal_handler);
    signal(SIGPIPE, signal_handler);
    signal(SIGTERM, signal_handler);
    can_sock_init();
    do {
        sleep(1);
        can_send_data(CAN_1, (unsigned char *)send_data, strlen(send_data) - 1);
    } while (!bexit);

    return 0;
}
